
#include once "fbimage\fbimage.bi"

#define SQUARE_WIDTH 40
#define SQUARE_HEIGHT 40
#define STEP_LENGTH 5
#define DEFAULT_PIECE_PLACEMENT "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR"
#define BOARD_WIDTH 320
#define BOARD_HEIGHT 320
#define PIECES_COUNT 32
#define NO_ANIMATION

namespace graph

dim shared as string resourcePath
dim shared as integer boardLeft
dim shared as integer boardTop
dim shared as boolean upsideDown = false

function XToScreen(x as integer) as integer
  return IIF(upsideDown, SQUARE_WIDTH * (8 - x) + boardLeft, SQUARE_WIDTH * (x - 1) + boardLeft)
end function

function YToScreen(y as integer) as integer
  return IIF(upsideDown, SQUARE_HEIGHT * (y - 1) + boardTop, SQUARE_HEIGHT * (8 - y) + boardTop)
end function

type TChessImage
  as any ptr fBackground, fForeground
  as integer fLeft, fTop
  declare sub Create()
  declare sub Reload(byval aFilename as string)
  declare sub Free()
  declare sub Redraw(byval aLeft as integer, byval aTop as integer)
  declare sub Hide()
end type

sub TChessImage.Create()
  fBackground = ImageCreate(SQUARE_WIDTH, SQUARE_HEIGHT)
  fForeground = ImageCreate(SQUARE_WIDTH, SQUARE_HEIGHT)
  fLeft = -1
  fTop = -1
end sub

sub TChessImage.Reload(byval aFilename as string)
  BLoad(resourcePath & aFilename, fForeground)
end sub

sub TChessImage.Free()
  ImageDestroy(fBackground)
  ImageDestroy(fForeground)
end sub

sub TChessImage.Redraw(byval aLeft as integer, byval aTop as integer)
  if fLeft <> -1 then
    Put (fLeft, fTop), fBackground, PSET
  end if
  fLeft = aLeft
  fTop = aTop
  Get (fLeft, fTop)-(fLeft + SQUARE_WIDTH - 1, fTop + SQUARE_HEIGHT - 1), fBackground
  Put (fLeft, fTop), fForeground, TRANS
end sub

sub TChessImage.Hide()
  Put (fLeft, fTop), fBackground, PSET
end sub

type TMovingChessImage
  as TChessImage img
  as integer fX, fY
  declare sub Create()
  declare sub Reload(byval aWhat as string)
  declare sub Free()
  declare sub Show overload(byval aX as integer, byval aY as integer)
  declare sub Show overload(byval aSquare as string)
  declare sub MoveTo(byval aX as integer, byval aY as integer)
end type

sub TMovingChessImage.Create()
  img.Create()
  fX = -1
  fY = -1
end sub

sub TMovingChessImage.Reload(byval aWhat as string)
  img.Reload(aWhat & ".bmp")
end sub

sub TMovingChessImage.Free()
  img.Free()
end sub

sub TMovingChessImage.Show overload(byval aX as integer, byval aY as integer)
  fX = aX
  fY = aY
  img.Redraw(XToScreen(fX), YToScreen(fY))
end sub

sub TMovingChessImage.Show overload(byval aSquare as string)
  dim as integer x, y
  x = aSquare[0] - Asc("a") + 1
  y = aSquare[1] - Asc("1") + 1
  Show(x, y)
end sub

sub TMovingChessImage.MoveTo(byval aX as integer, byval aY as integer)
  dim as integer dx, dy, sx, sy
#ifdef NO_ANIMATION
  fX = aX
  fY = aY
  sx = XToScreen(fX)
  sy = YToScreen(fY)
  img.Redraw(sx, sy)
#ELSE
  dx = sgn(aX - fX)
  dy = sgn(aY - fY)
  if Abs(aX - fX) = 2 * Abs(aY - fY) then dx *= 2
  if Abs(aY - fY) = 2 * Abs(aX - fX) then dy *= 2
  if upsideDown then
    dx *= -1
    dy *= -1
  end if
  sx = XToScreen(fX)
  sy = YToScreen(fY)
  fX = aX
  fY = aY
  dx *= STEP_LENGTH
  dy *= STEP_LENGTH
  do
    Sleep(10, 1)
    sx += dx
    sy -= dy
    img.Redraw(sx, sy)
  loop until (sx = XToScreen(fX)) and (sy = YToScreen(fY))
#endif
end sub

type TGraphicalChessboard
  dim array(PIECES_COUNT - 1) as TMovingChessImage
  dim what (PIECES_COUNT - 1) as string * 2
  dim where(PIECES_COUNT - 1) as string * 2
  dim number as integer = PIECES_COUNT
  dim fBackup as any ptr
  dim fValid as any ptr
  declare sub Create()
  declare sub Free()
  declare sub ChangePieces(byval aPlacement as string = DEFAULT_PIECE_PLACEMENT)
  declare sub RedrawPieces()
  declare sub ClearBoard()
  declare sub MovePiece overload( _
    byval aX1 as integer, _
    byval aY1 as integer, _
    byval aX2 as integer, _
    byval aY2 as integer _
 )
  declare sub MovePiece overload(byval move as string)
  declare function HidePiece overload( _
    byval aX1 as integer, _
    byval aY1 as integer _
 ) as integer
  declare function HidePiece overload(byval square as string) as integer
  declare sub Promote(byval aWhere as string, byval aNewPieceSymbol as byte)
  declare sub Highlight overload( _
    byval aX1 as integer, _
    byval aY1 as integer _
 )
   declare sub Highlight overload( _
    byval square as string _
 )
 declare sub Backup()
 declare sub Restore_()
end type

sub TGraphicalChessboard.Create()
  dim board as any ptr = ImageCreate(BOARD_WIDTH, BOARD_HEIGHT)
  BLoad resourcePath & "board.bmp", board
  Put (boardLeft, boardTop), board, TRANS
  ImageDestroy board
  for i as integer = 0 to number - 1
    array(i).Create()
  next i
  fBackup = ImageCreate(BOARD_WIDTH, BOARD_HEIGHT)
  fValid = LoadRGBAFile(resourcePath & "valid.png")
end sub

sub TGraphicalChessboard.Free()
  for i as integer = 0 to number - 1
    array(i).Free()
  next i
  ImageDestroy fBackup
  ImageDestroy fValid
end sub

sub TGraphicalChessboard.ChangePieces(byval aPlacement as string = DEFAULT_PIECE_PLACEMENT)
  dim as byte _
  bs = Asc("/"), _
  b1 = Asc("1"), _
  b8 = Asc("8"), _
  ba = Asc("a")
  dim as integer x = 1, y = 8, j = 0
  dim n as byte
  for i as integer = 0 to Len(aPlacement) - 1
    if aPlacement[i] = bs then
      x = 1
      y -= 1
    else
      if (aPlacement[i] >= b1) and (aPlacement[i] <= b8) then
        n = aPlacement[i] - b1 + 1
        do while n >= 1
          x += 1
          n -= 1
        loop
      else
        what(j) = IIF(aPlacement[i] < ba, "w", "b") & LCase(Chr(aPlacement[i]))
        where(j) = Chr(x + ba - 1, y + b1 - 1)
        array(j).Reload(what(j))
        x += 1
        j += 1
      end if
    end if
  next i
  number = j
end sub

sub TGraphicalChessboard.RedrawPieces()
  for i as integer = 0 to number - 1
    array(i).Show(where(i))
  next i
end sub

sub TGraphicalChessboard.ClearBoard()
  for i as integer = 0 to number - 1
    array(i).img.Hide()
  next i
end sub

sub TGraphicalChessboard.MovePiece overload( _
  byval aX1 as integer, _
  byval aY1 as integer, _
  byval aX2 as integer, _
  byval aY2 as integer _
)
  dim as integer i, found = 0, res = 1
  
  i = 0
  while (i <= PIECES_COUNT - 1) and (found = 0)
    if (array(i).fX = aX2) and (array(i).fY = aY2) then
      found = 1
      array(i).img.Hide()
      array(i).fX = -1
      array(i).fY = -1
      where(i) = ".."
    else
      i += 1
    end if
  wend
  
  i = 0
  while (i <= PIECES_COUNT - 1) and (res = 1)
    if (array(i).fX = aX1) and (array(i).fY = aY1) then
      res = 0
    else
      i += 1
    end if
  wend
  
  if res = 0 then
    array(i).MoveTo(aX2, aY2)
    where(i) = Chr(aX2 + Asc("a") - 1, aY2 + Asc("1") - 1)
  end if
end sub

sub TGraphicalChessboard.MovePiece overload(byval move as string)
  move = LCase(move)
  MovePiece( _
    move[0] - Asc("a") + 1, _
    move[1] - Asc("1") + 1, _
    move[2] - Asc("a") + 1, _
    move[3] - Asc("1") + 1 _
 )
end sub

function TGraphicalChessboard.HidePiece overload( _
  byval aX1 as integer, _
  byval aY1 as integer _
) as integer
  dim as integer i, bx, by, res = 1
  i = 0
  while (i <= PIECES_COUNT - 1) and (res = 1)
    if (array(i).fX = aX1) and (array(i).fY = aY1) then
      res = 0
    else
      i += 1
    end if
  wend
  if res = 0 then
    array(i).img.Hide()
    array(i).fX = -1
    array(i).fY = -1
  end if
  return res
end function

function TGraphicalChessboard.HidePiece overload(byval square as string) as integer
  square = LCase(square)
  return HidePiece( _
    square[0] - Asc("a") + 1, _
    square[1] - Asc("1") + 1 _
 )
end function

sub TGraphicalChessboard.Promote(byval aWhere as string, byval aNewPieceSymbol as byte)
  dim i as integer = 0
  do while (i <= number - 1) andalso (where(i) <> aWhere)
    i += 1  
  loop
  if i <= number - 1 then
    array(i).img.Hide()
    what(i)[1] = aNewPieceSymbol
    array(i).img.Reload(what(i) & ".bmp")
    array(i).Show(where(i))
  end if
end sub

sub TGraphicalChessboard.Highlight overload(_
    byval aX1 as integer, _
    byval aY1 as integer _
 )
   Put (XToScreen(aX1), YToScreen(aY1)), fValid, ALPHA
 end sub  

sub TGraphicalChessboard.Highlight overload(_
    byval square as string _
 )
   Highlight(square[0] - Asc("a") + 1, square[1] - Asc("1") + 1)
 end sub
 
sub TGraphicalChessboard.Backup()
  Get (boardLeft, boardTop)-(boardLeft + BOARD_WIDTH - 1, boardTop + BOARD_HEIGHT - 1), fBackup
end sub

sub TGraphicalChessboard.Restore_()
  Put (boardLeft, boardTop), fBackup, PSET
end sub

end namespace
